/*
 * Part of the GaBIEn project.
 * (Graphics and Basic Input Engine)
 *
 *        DO WHATEVER YOU WANT TO PUBLIC LICENSE 
 *                    Version 2, December 2004 
 *
 * Modified by gamemanj <asdd2808@gmail.com> to remove expletives
 * Original copyright (C) 2004 Sam Hocevar <sam@hocevar.net> 
 *
 * Everyone is permitted to copy and distribute verbatim or modified 
 * copies of this license document, and changing it is allowed as long 
 * as the name is changed. 
 *
 *            DO WHATEVER YOU WANT TO PUBLIC LICENSE 
 *   TERMS AND CONDITIONS FOR COPYING, DISTRIBUTION AND MODIFICATION 
 *
 *  0. You just DO WHATEVER YOU WANT TO.
 */
package gabien;

/**
 * A graphics and input driver.The idea is to speed up work on ports.
 * @author matt
 */
public interface IGrInDriver {
    //ASCII keys.
    public static final int VK_ENTER = 10;
    public static final int VK_BACK_SPACE = 8;
    public static final int VK_TAB = 9;
    public static final int VK_CANCEL = 3;
    public static final int VK_CLEAR = 12;
    public static final int VK_SHIFT = 16;
    public static final int VK_CONTROL = 17;
    public static final int VK_ALT = 18;
    public static final int VK_PAUSE = 19;
    public static final int VK_CAPS_LOCK = 20;
    public static final int VK_ESCAPE = 27;
    public static final int VK_SPACE = 32;
    public static final int VK_PAGE_UP = 33;
    public static final int VK_PAGE_DOWN = 34;
    public static final int VK_END = 35;
    public static final int VK_HOME = 36;
    public static final int VK_LEFT = 37;
    public static final int VK_UP = 38;
    public static final int VK_RIGHT = 39;
    public static final int VK_DOWN = 40;
    public static final int VK_COMMA = 44;
    public static final int VK_MINUS = 45;
    public static final int VK_PERIOD = 46;
    public static final int VK_SLASH = 47;
    public static final int VK_0 = 48;
    public static final int VK_1 = 49;
    public static final int VK_2 = 50;
    public static final int VK_3 = 51;
    public static final int VK_4 = 52;
    public static final int VK_5 = 53;
    public static final int VK_6 = 54;
    public static final int VK_7 = 55;
    public static final int VK_8 = 56;
    public static final int VK_9 = 57;
    public static final int VK_SEMICOLON = 59;
    public static final int VK_EQUALS = 61;
    public static final int VK_A = 65;
    public static final int VK_B = 66;
    public static final int VK_C = 67;
    public static final int VK_D = 68;
    public static final int VK_E = 69;
    public static final int VK_F = 70;
    public static final int VK_G = 71;
    public static final int VK_H = 72;
    public static final int VK_I = 73;
    public static final int VK_J = 74;
    public static final int VK_K = 75;
    public static final int VK_L = 76;
    public static final int VK_M = 77;
    public static final int VK_N = 78;
    public static final int VK_O = 79;
    public static final int VK_P = 80;
    public static final int VK_Q = 81;
    public static final int VK_R = 82;
    public static final int VK_S = 83;
    public static final int VK_T = 84;
    public static final int VK_U = 85;
    public static final int VK_V = 86;
    public static final int VK_W = 87;
    public static final int VK_X = 88;
    public static final int VK_Y = 89;
    public static final int VK_Z = 90;
    public static final int VK_OPEN_BRACKET = 91;
    public static final int VK_BACK_SLASH = 92;
    public static final int VK_CLOSE_BRACKET = 93;
    public static final int VK_NUMPAD0 = 96;
    public static final int VK_NUMPAD1 = 97;
    public static final int VK_NUMPAD2 = 98;
    public static final int VK_NUMPAD3 = 99;
    public static final int VK_NUMPAD4 = 100;
    public static final int VK_NUMPAD5 = 101;
    public static final int VK_NUMPAD6 = 102;
    public static final int VK_NUMPAD7 = 103;
    public static final int VK_NUMPAD8 = 104;
    public static final int VK_NUMPAD9 = 105;
    public static final int VK_MULTIPLY = 106;
    public static final int VK_ADD = 107;
    public static final int VK_SEPARATER = 108;
    public static final int VK_SEPARATOR = 108;
    public static final int VK_SUBTRACT = 109;
    public static final int VK_DECIMAL = 110;
    public static final int VK_DIVIDE = 111;
    public static final int VK_DELETE = 127;

    //BASIC
    public static interface IImage {
        public int GetWidth();
        public int GetHeight();
    }
    public boolean StillRunning();
    public int GetWidth();
    public int GetHeight();
    public IImage GetImage(String a,int r,int g,int b);
    public void BlitImage(int srcx,int srcy,int srcw,int srch,int x,int y,IImage i);

    //Display 8 pixel high text in Monospace.
    //Highly unreliable.Only use for debugging.
    public void Display8pxText(int x,int y,int r,int g,int b,String text);
    
    //'but colour key'
    public void BlitBCKImage(int srcx,int srcy,int srcw,int srch,int x,int y,IImage i);
    public void Clear(int i, int i0, int i1);
    public void Flush();//This copies the IsKeyDown buffer to WasKeyDown.
    //INPUT-BASIC
    public boolean IsKeyDown(int KEYID);
    public boolean IsKeyJustPressed(int KEYID);//Note that if this returns true,it won't return true again for any other call for that key until another press happens.
    public void ClearKeys();//This will clear all key-related buffers.
                            //This includes the Typist extension's keybuffer,and the mouse.
    //INPUT-ADVANCED
    //Note that these only apply when the mouse is down.
    public int GetMouseX();
    public int GetMouseY();
    public boolean GetMouseDown();
    public boolean GetMouseJustDown();//Note that if this returns true,it won't return true again for any other call until another press happens.
    public int GetMouseButton();//Get the Mouse button currently pressed. 0:None 1:Left,3:Right
    //if it's completely impossible to provide 320x240,or the player requests it,put the true values in and hope it works.
    //INPUT-TYPIST
    //Typist allows the use of Typed events.
    //Thus,it's good to implement it.

    //This can show a keyboard covering the lower half of the screen or less.
    //If one is already up,don't show it again.
    
    public void SetTypeBuffer(String s);
    public String GetTypeBuffer();
    public void Shutdown();
}
