/*
 * Part of the GaBIEn project.
 * (Graphics and Basic Input Engine)
 *
 *        DO WHATEVER YOU WANT TO PUBLIC LICENSE 
 *                    Version 2, December 2004 
 *
 * Modified by gamemanj <asdd2808@gmail.com> to remove expletives
 * Original copyright (C) 2004 Sam Hocevar <sam@hocevar.net> 
 *
 * Everyone is permitted to copy and distribute verbatim or modified 
 * copies of this license document, and changing it is allowed as long 
 * as the name is changed. 
 *
 *            DO WHATEVER YOU WANT TO PUBLIC LICENSE 
 *   TERMS AND CONDITIONS FOR COPYING, DISTRIBUTION AND MODIFICATION 
 *
 *  0. You just DO WHATEVER YOU WANT TO.
 */

package gabien;

import gabien.ISoundDriver;
import java.io.ByteArrayOutputStream;
import java.io.DataOutputStream;
import java.io.IOException;
import java.util.LinkedList;
import javax.sound.sampled.AudioFormat;
import javax.sound.sampled.AudioSystem;
import javax.sound.sampled.LineUnavailableException;
import javax.sound.sampled.SourceDataLine;

/**
 * Note that it may be a good idea to move part of this into GaBIEn-Common,
 * and make ISoundDriver take raw data.
 * @author matt
 */
final class SoundDriver implements ISoundDriver,Runnable {
    SourceDataLine sdl;
    public SoundDriver() throws LineUnavailableException
    {
        AudioFormat af=new AudioFormat(22050,16,2,true,true);
        sdl=AudioSystem.getSourceDataLine(af);
        sdl.open(af,2205);
        soundthread.start();
    }
    private Thread soundthread=new Thread(this);
    private JAVAX_Channel[] channels=new JAVAX_Channel[128];

    @Override
    public void Shutdown() {
        alive=false;
    }
    
    public static class JAVAX_Channel implements IChannel
    {
        short[] data;
        int pos=0;
        double VL,VR;
        
        public short Get()
        {
            try {
            if (data==null) return 0;
            
            short a=data[pos++];
            if (pos==data.length) pos=0;
            return a;
            } catch (Exception e) {
                pos=0;
                return 0;
            }
        }
        //This is the first time I have ever written this routine.Ever.
        public void ChPitch(double ratio,short[] data2)
        {
            if (ratio<=0) {data2=new short[0]; return;}
            double position=0;
            double adv=ratio/data2.length;
            LinkedList<Short> shortdata=new LinkedList<Short>();
            while (position<1.0)
            {
                int ii=(int)Math.floor(position*data2.length);
                int ii2=(int)Math.ceil(position*data2.length);
                while (ii<0) ii+=data2.length;
                while (ii>=data2.length) ii-=data2.length;
                while (ii2<0) ii2+=data2.length;
                while (ii2>=data2.length) ii2-=data2.length;
                shortdata.add(lerp(data2[ii],data2[ii2],(position*data2.length)-ii));
                position+=adv;
            }
            data=new short[shortdata.size()];
            for (int p=0;p<data.length;p++) data[p]=shortdata.get(p);
        }
        public JAVAX_Channel()
        {
            
        }
        public short Scale(short a,double V)
        {
            double b=a*V;
            if (b>32767) b=32767;
            if (b<-32768) b=-32768;
            return (short)b;
        }
        public short[] CreateData(int amount)
        {
            short[] s=new short[amount*2];
            for (int px=0;px<amount;px++)
            {
                    short V=Get();
                    s[(px*2)+0]=Scale(V,VL);
                    s[(px*2)+1]=Scale(V,VR);
            }
            return s;
        }

        @Override
        public void PlaySound(double Pitch, double VolL, double VolR, short[] sound) {
            VL=VolL;
            VR=VolR;
            ChPitch(Pitch,sound);
            pos=0;
        }

        @Override
        public void SetVolume(double VolL, double VolR) {
            VL=VolL;
            VR=VolR;
        }

        @Override
        public double VolL() {
            return VL;
        }

        @Override
        public double VolR() {
            return VR;
        }

        private Short lerp(short s, short s0, double d) {
            int diff=s0-s;
            diff*=d;
            return (short)(s+diff);
        }

    }
    @Override
    public IChannel CreateChannel() {
            JAVAX_Channel jc=new JAVAX_Channel();
            for (int p=0;p<channels.length;p++)
            {
                if (channels[p]==null)
                {
                    channels[p]=jc;
                    break;
                }
            }
            return jc;
    }
    public byte[] CreateData(int amount)
    {
                try {
            ByteArrayOutputStream baos=new ByteArrayOutputStream();
            DataOutputStream dos=new DataOutputStream(baos);
            for (int px=0;px<amount;px++)
            {
                    int L=0;
                    int R=0;
                    for (JAVAX_Channel js : channels)
                    {
                        if (js==null) continue;
                    short[] r=js.CreateData(1);
                    L+=r[0];
                    R+=r[1];
                    }
                    if (L>32767) L=32767;
                    if (L<-32768) L=-32768;
                    if (R>32767) R=32767;
                    if (R<-32768) R=-32768;
                    dos.writeShort(L);
                    dos.writeShort(R);
            }
            dos.flush();
            byte[] bytes=baos.toByteArray();
            return bytes;
                } catch (IOException ex) {
                    return null;
                }
    }
    private boolean alive=true;
    @Override
    public void run()
    {
        int buf=2205;
        while (alive)
        {
            int a=(sdl.available()/4);
            while (a>buf) {
                Thread.yield();
            a=(sdl.available()/4);
            }
            if (a<=0)
            {
                continue;
            }
            byte[] bytes=CreateData(a);
            
            sdl.write(bytes, 0, bytes.length);
            if (!sdl.isRunning())
            {
                System.err.println("SOUND:needed restart...");
            sdl.start();
            }
        }
        
    }
    @Override
    public void DeleteChannel(IChannel ic) {
        
        for (int p=0;p<channels.length;p++)
        {
            if (channels[p]==ic) channels[p]=null;
        }
    }
    
}
