/*
 * Part of the JavaChan project.
 * (Java ikaChan)
 *
 *        DO WHATEVER YOU WANT TO PUBLIC LICENSE 
 *                    Version 2, December 2004 
 *
 * Modified by gamemanj <asdd2808@gmail.com> to remove expletives
 * Original copyright (C) 2004 Sam Hocevar <sam@hocevar.net> 
 *
 * Everyone is permitted to copy and distribute verbatim or modified 
 * copies of this license document, and changing it is allowed as long 
 * as the name is changed. 
 *
 *            DO WHATEVER YOU WANT TO PUBLIC LICENSE 
 *   TERMS AND CONDITIONS FOR COPYING, DISTRIBUTION AND MODIFICATION 
 *
 *  0. You just DO WHATEVER YOU WANT TO.
 */
package javachan;

import gabien.IGrInDriver;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import javachan.npcs.AttackStarNPCType;
import javachan.npcs.BasicNPCType;
import javachan.npcs.CrabNPCType;
import javachan.npcs.GlobefishNPCType;
import javachan.npcs.IsoginNPCType;
import javachan.npcs.SleepNPCType;

/**
 *
 * @author matt
 */
public class GameStateNPChar {
    public NPCCharacter[] chars = new NPCCharacter[100];
    public BasicNPCType[] npctypes=new BasicNPCType[] {
        new GlobefishNPCType(),
        new IsoginNPCType(),
        new CrabNPCType(),
        new SleepNPCType(),
        new BasicNPCType(),
        new AttackStarNPCType(),
        new BasicNPCType(),
        new BasicNPCType(),
        new BasicNPCType(),
        new BasicNPCType()};
    public static class NPCCharacter implements IPhysicsObject {
        public boolean Exists=false;
        public double X, Y,VX,VY, OX, OY;
        public int Type, CollisionType,Status,Status2, EventID;//collisiontype:0:NULL 1:BLOCK 2:ENEMY 3:EVENT
        public boolean OnGround=false;
        //STATUS2,ONGROUND,LAST TIME IKACHAN ANNOYED ME ARE NOT SAVED.
        public NPCCharacter() {
        }

        @Override
        public Rect GetRectangle() {
            return new Rect((int) (X * 16), (int) (Y * 16), 16, 16);
        }
        public boolean StopsIkachan() {
            return (CollisionType == 3) || (CollisionType == 2);
        }
        //Called once per frame at most on collision with Ikachan.
        //Time is the current 'physical time'-essentially the amount of seconds not spent in a script.
        //This is used to avoid Ikachan continually activating the same script.
        public void HandleIkachanCollision(GameState gs,IEventCallback iec, double Time) {
            //TODO:Anything involving Ironhead.
            //TODO:HandleIkachanCollision for enemies.
            if (CollisionType == 0) {
                if (gs.LastAnnoyTime + 1 <= Time) {
                    iec.StartEvent(EventID);
                    gs.LastAnnoyTime = Time;
                }
            }
            if (CollisionType == 3) {
                if (gs.LastAnnoyTime + 1 <= Time) {
                    iec.StartEvent(EventID);
                    gs.LastAnnoyTime = Time;
                }
            }
        }

        @Override
        public double GetX() {
            return X;
        }

        @Override
        public double GetY() {
            return Y;
        }

        @Override
        public double GetVX() {
            return VX;
        }

        @Override
        public double GetVY() {
            return VY;
       }

        @Override
        public void SetX(double X) {
            this.X=X;
        }

        @Override
        public void SetY(double Y) {
            this.Y=Y;
        }

        @Override
        public void SetVX(double VX) {
            this.VX=VX;
        }

        @Override
        public void SetVY(double VY) {
            this.VY=VY;
        }
    }
    //Note that there MAY be a problem with this.
    //Or the function below.
    //One hint:It involves the sign bit.
    //Not a major problem,but feel free to fix it if you know the 'right' value.
    private static void DoubleToFixedPoint(double d, FileOutputStream fos) throws IOException {
        int dd=(int)Math.floor(d);
        dd=dd<<14;
        double fractional=d-Math.floor(d);
        fractional*=1<<14;
        dd+=(int) fractional;
        fos.write((dd&0xFF));
        fos.write((dd&0xFF00)>>8);
        fos.write((dd&0xFF0000)>>16);
        fos.write((dd&0xFF000000)>>24);
    }

    private static double FixedPointToDouble(InputStream is) throws IOException {
        double fractional = is.read();
        int mb = is.read();//WWFFFFFF
        fractional += (mb & 0x3F) << 8;
        fractional /= (1 << 14);
        int whole = (mb & (64 | 128)) / 64;
        whole |= is.read() << 2;
        whole |= is.read() << 10;
        return fractional + whole;
    }

    
    public GameStateNPChar()
    {
        for (int p = 0; p < chars.length; p++) {
            chars[p]=new NPCCharacter();
        }
    }
    
    
    public void Load(InputStream is) throws Exception {
        for (int p = 0; p < chars.length; p++) {
            NPCCharacter nc = new NPCCharacter();
            boolean sw = is.read() != 0;
            nc.CollisionType = is.read();
            nc.Type = is.read();
            nc.EventID = is.read();
            nc.Status=is.read();//?
            nc.Status |= is.read() << 8;
            nc.X = FixedPointToDouble(is);
            nc.Y = FixedPointToDouble(is);
            nc.OX = FixedPointToDouble(is) - nc.X;//not 'supposed' to do this,but...
            nc.OY = FixedPointToDouble(is) - nc.Y;
            nc.Exists=sw;
            chars[p] = nc;
        }
    }
   
    public void Save(FileOutputStream fos) throws IOException {
        for (int p = 0; p < chars.length; p++) {
            NPCCharacter nc=chars[p];
            if (nc.Exists)
            { 
                fos.write(1);
            } else {
                fos.write(0);
            }
            fos.write(nc.CollisionType);
            fos.write(nc.Type);
            fos.write(nc.EventID);
            fos.write(nc.Status&0xFF);
            fos.write((nc.Status&0xFF00)>>8);
            DoubleToFixedPoint(nc.X,fos);
            DoubleToFixedPoint(nc.Y,fos);
            DoubleToFixedPoint(nc.X+nc.OX,fos);
            DoubleToFixedPoint(nc.Y+nc.OY,fos);
        }
    }

    public void Render(int px, int py, double Time,GameState gs,IGrInDriver gd) {
        for (int p = 0; p < chars.length; p++) {
            if (chars[p].Exists)
            npctypes[chars[p].Type].Draw(px, py,Time,chars[p],gs,gd);
        }
    }
}
