/*
 * Part of the JavaChan project.
 * (Java ikaChan)
 *
 *        DO WHATEVER YOU WANT TO PUBLIC LICENSE 
 *                    Version 2, December 2004 
 *
 * Modified by gamemanj <asdd2808@gmail.com> to remove expletives
 * Original copyright (C) 2004 Sam Hocevar <sam@hocevar.net> 
 *
 * Everyone is permitted to copy and distribute verbatim or modified 
 * copies of this license document, and changing it is allowed as long 
 * as the name is changed. 
 *
 *            DO WHATEVER YOU WANT TO PUBLIC LICENSE 
 *   TERMS AND CONDITIONS FOR COPYING, DISTRIBUTION AND MODIFICATION 
 *
 *  0. You just DO WHATEVER YOU WANT TO.
 */
package javachan.games;

import java.util.LinkedList;
import java.util.List;
import javachan.GameState;
import javachan.GameStateMap;
import javachan.GameStateNPChar;
import javachan.IEventCallback;
import javachan.IPhysicsEngine;
import javachan.IPhysicsObject;
import javachan.IntegerPoint;
import javachan.Rect;
import javachan.blocks.BasicBlockType;
import javachan.bosses.BasicBoss;

/**
 * "UNLESS someone like you cares a whole awful lot,nothing will ever change."
 * Makes about as much sense to people who like NXEngine for Ikachan,
 * as it does to anything else.
 * 
 * @author matt
 */
public class PhysicsImplementation implements IPhysicsEngine {

    public PhysicsImplementation() {
    }
    //Note that it's the PC collisions that create events.This is important.
    //Furthermore,the whole Capacitor thing is one massive special case-
    //-1:Not needed,0:Left,1:Right,2:Up,3:Down

    private int PushOutObjectFromObject(IPhysicsObject po, Rect r) {
        if (r.intersects(po.GetRectangle())) {
            double oX = po.GetX();
            double oY = po.GetY();
            int p0 = PushOutObjectPass(po, r, -1, 0);
            po.SetX(oX);
            po.SetY(oY);
            int p1 = PushOutObjectPass(po, r, 1, 0);
            po.SetX(oX);
            po.SetY(oY);
            int p2 = PushOutObjectPass(po, r, 0, -1);
            po.SetX(oX);
            po.SetY(oY);
            int p3 = PushOutObjectPass(po, r, 0, 1);
            po.SetX(oX);
            po.SetY(oY);
            //Okay,now find which is better.
            int bestind = 2;
            int besttime = 1024;
            int[] arr = new int[]{p0, p1, p2, p3};
            for (int s = 0; s < 4; s++) {
                if (arr[s] < besttime) {
                    besttime = arr[s];
                    bestind = s;
                }
            }
            if (bestind == 0) {
                PushOutObjectPass(po, r, -1, 0);
            }
            if (bestind == 1) {
                PushOutObjectPass(po, r, 1, 0);
            }
            if (bestind == 2) {
                PushOutObjectPass(po, r, 0, -1);
            }
            if (bestind == 3) {
                PushOutObjectPass(po, r, 0, 1);
            }
            return bestind;
        }
        return -1;
    }

    private int PushOutObjectFromWorld(IPhysicsObject po, GameStateMap r) {
        double oX = po.GetX();
        double oY = po.GetY();
        int p0 = PushOutWorldPass(po, r, -1, 0);
        po.SetX(oX);
        po.SetY(oY);
        int p1 = PushOutWorldPass(po, r, 1, 0);
        po.SetX(oX);
        po.SetY(oY);
        int p2 = PushOutWorldPass(po, r, 0, -1);
        po.SetX(oX);
        po.SetY(oY);
        int p3 = PushOutWorldPass(po, r, 0, 1);
        po.SetX(oX);
        po.SetY(oY);
        //Okay,now find which is better.
        int bestind = 0;
        int besttime = Integer.MAX_VALUE;
        int[] arr = new int[]{p0, p1, p2, p3};
        for (int s = 0; s < 4; s++) {
            if (arr[s] < besttime) {
                besttime = arr[s];
                bestind = s;
            }
        }
        if (bestind == 0) {
            PushOutWorldPass(po, r, -1, 0);
        }
        if (bestind == 1) {
            PushOutWorldPass(po, r, 1, 0);
        }
        if (bestind == 2) {
            PushOutWorldPass(po, r, 0, -1);
        }
        if (bestind == 3) {
            PushOutWorldPass(po, r, 0, 1);
        }
        if (arr[bestind]>16)
        {
            System.out.println("REALLY BAD! Potentially warping object!");
        }
        return bestind;
    }

    public int Clamp(int n, int min, int max) {
        if (n < min) {
            n = min;
        }
        if (n > max) {
            n = max;
        }
        return n;
    }

    public int RunPushOutOfMapPass(IPhysicsObject op, GameState gs, boolean soft) {
        GameStateMap map = gs.map;
        int cx = (int) op.GetX();
        int cl = cx - 2;
        int cr = cx + 2;
        cl = Clamp(cl, 0, map.Width - 1);
        cr = Clamp(cr, 0, map.Width - 1);
        int cy = (int) op.GetY();
        int cu = cy - 2;
        int cd = cy + 2;
        cu = Clamp(cu, 0, map.Height - 1);
        cd = Clamp(cd, 0, map.Height - 1);
        List<IntegerPoint> Blocks = new LinkedList<IntegerPoint>();
        for (int px = cl; px < cr; px++) {
            for (int py = cu; py < cd; py++) {
                int md = map.data[px + (py * map.Width)];
                BasicBlockType bbt = map.blocktypes[md / 16];
                if (bbt != null) {
                    Rect br = bbt.GetCollision(px, py, md % 16);
                    if (br.intersects(op.GetRectangle())) {
                        if (bbt.CollisionStops() || soft) {
                            IntegerPoint ip = new IntegerPoint();
                            ip.X = px;
                            ip.Y = py;
                            Blocks.add(ip);
                        }
                    }
                }
            }
        }
        if (Blocks.size() > 0) {
            if (soft) {
                for (IntegerPoint ip : Blocks) {
                    int md = map.data[ip.X + (ip.Y * map.Width)];
                    BasicBlockType bbt = map.blocktypes[md / 16];
                    bbt.HandleSoftCollision(ip.X, ip.Y, gs, op);
                }
            } else {
                int d = PushOutObjectFromWorld(op, map);
                for (IntegerPoint ip : Blocks) {
                    int md = map.data[ip.X + (ip.Y * map.Width)];
                    BasicBlockType bbt = map.blocktypes[md / 16];
                    bbt.HandleHardCollision(ip.X, ip.Y, gs, op, d);
                }
                return d;
            }
        }
        return -1;
    }

    public int RunPushOutOfObjectPass(IPhysicsObject op, GameState gs, double Time, IEventCallback iec) {
        GameStateNPChar map = gs.npcs;
        for (int px = 0; px < map.chars.length; px++) {
            if (map.chars[px] == op) {
                continue;
            }
            if (!map.chars[px].Exists) {
                continue;
            }
            //collisiontype:0:NULL 1:BLOCK 2:ENEMY 3:EVENT
            int ctype = map.chars[px].CollisionType;
            boolean Colliding = map.chars[px].GetRectangle().intersects(op.GetRectangle());
            if (Colliding) {
                if ((ctype == 0) || (ctype == 3)) {
                    if (op == gs) {
                        map.chars[px].HandleIkachanCollision(gs, iec, Time);
                    }
                }
                if ((ctype == 2) || (ctype == 3)) {
                    int r = PushOutObjectFromObject(op, map.chars[px].GetRectangle());
                    if (gs == op) {
                        if (ctype == 2) {
                            map.npctypes[map.chars[px].Type].IkachanTouchedEnemy(map.chars[px], gs, r, iec);
                        }
                    }
                    if ((r == 3) || (r == 2)) {
                        op.SetVY(0);
                    }
                    if ((r == 1) || (r == 0)) {
                        op.SetVX(0);
                    }
                    return r;
                }
            }
        }
        return -1;
    }

    private boolean[] ComputeGaf(IPhysicsObject op, GameState gs, double Time, IEventCallback iec,boolean firstpass) {
        boolean[] ret = new boolean[4];
        boolean IsIkachan = (op == gs);

        if (IsIkachan) {
            int r = RunPushOutOfObjectPass(op, gs, Time, iec);
            if (r != -1) {
                ret[r] = true;
            }
        }
        int r = RunPushOutOfMapPass(op, gs, false);
        if (r != -1) {
            ret[r] = true;
        }
        if (firstpass)
        RunPushOutOfMapPass(op, gs, true);
        if (firstpass)
        if (IsIkachan) {
            for (BasicBoss boss : gs.Bosses)
            {
                if (boss.Exists(gs))
                {
                    int hitironhead = PushOutObjectFromObject(op, boss.GetRectangle());
                    if (hitironhead!=-1) {
                        ret[hitironhead]=true;
                        boss.Hit(gs, hitironhead, iec, Time);
                    }
                }
            }
        }

        return ret;
    }

    private int PushOutObjectPass(IPhysicsObject po, Rect r, int i, int i0) {
        int time = 0;
        while (po.GetRectangle().intersects(r)) {
            po.SetX(po.GetX() + (i / 16.0));
            po.SetY(po.GetY() + (i0 / 16.0));
            time++;
        }
        return time;
    }

    private int PushOutWorldPass(IPhysicsObject po, GameStateMap map, int i, int i0) {
        int time = 0;
        while (true) {
            int cx = (int)Math.floor(po.GetX());
            int cl = cx - 2;
            int cr = cx + 2;
            cl = Clamp(cl, 0, map.Width - 1);
            cr = Clamp(cr, 0, map.Width - 1);
            int cy = (int)Math.floor(po.GetY());
            int cu = cy - 2;
            int cd = cy + 2;
            cu = Clamp(cu, 0, map.Height - 1);
            cd = Clamp(cd, 0, map.Height - 1);
            boolean intersected = false;
            for (int px = cl; px < cr; px++) {
                for (int py = cu; py < cd; py++) {
                    int md = map.data[px + (py * map.Width)];
                    BasicBlockType bbt = map.blocktypes[md / 16];
                    if (bbt != null) {
                        Rect br = bbt.GetCollision(px, py, md % 16);
                        if (br.intersects(po.GetRectangle())) {
                            if (bbt.CollisionStops()) {
                                intersected = true;
                            }
                        }
                    }
                }
            }
            if (!intersected) {
                break;
            }
            po.SetX(po.GetX() + (i / 16.0));
            po.SetY(po.GetY() + (i0 / 16.0));
            time++;
        }
        return time;
    }

    @Override
    public boolean[] Calculate(IEventCallback ev, IPhysicsObject ipo, GameState gs, double InGameTime) {
        int passes=1;
        while ((Math.abs(ipo.GetVX()/passes)+Math.abs(ipo.GetVY()/passes))>0.025)
        {
            passes*=2;
        }
        boolean[] aa=new boolean[4];
        for (int a=0;a<passes;a++)
        {
            ipo.SetX(ipo.GetX()+(ipo.GetVX()/passes));
            boolean[] b=ComputeGaf(ipo, gs, InGameTime, ev,false);
            CancelVelocityInDirections(b,ipo);
            aa[0]=aa[0]||b[0];
            aa[1]=aa[1]||b[1];
            aa[2]=aa[2]||b[2];
            aa[3]=aa[3]||b[3];
            ipo.SetY(ipo.GetY()+(ipo.GetVY()/passes));
            b=ComputeGaf(ipo, gs, InGameTime, ev,false);
            CancelVelocityInDirections(b,ipo);
            aa[0]=aa[0]||b[0];
            aa[1]=aa[1]||b[1];
            aa[2]=aa[2]||b[2];
            aa[3]=aa[3]||b[3];
            b=ComputeGaf(ipo, gs, InGameTime, ev,a==0);
            aa[0]=aa[0]||b[0];
            aa[1]=aa[1]||b[1];
            aa[2]=aa[2]||b[2];
            aa[3]=aa[3]||b[3];
            CancelVelocityInDirections(b,ipo);
        }
        return aa;
    }
    //0:Left,1:Right,2:Up,3:Down
    //Note that these directions are the directions that the player was pushed in to move them out of a block.
    private void CancelVelocityInDirections(boolean[] dirs,IPhysicsObject op)
    {
        if (dirs[0])
            op.SetVX(0);
        if (dirs[1])
            op.SetVX(0);
        if (dirs[2])
            op.SetVY(0);
        if (dirs[3])
            op.SetVY(0);
    }
}
